from zope.interface import implements
from zope.component import getUtility
from zope.component import queryUtility

from zope.app.container.interfaces import INameChooser

from plone.app.content.interfaces import INameFromTitle
from plone.i18n.normalizer.interfaces import IUserPreferredURLNormalizer
from plone.i18n.normalizer.interfaces import IURLNormalizer

from Acquisition import aq_inner, aq_parent, aq_base
from Products.CMFCore.interfaces import IPropertiesTool

ATTEMPTS = 1000

class NormalizingNameChooser(object):
    """A name chooser for a Zope object manager.
    
    If the object is adaptable to or provides INameFromTitle, use the
    title to generate a name.
    """
    
    implements(INameChooser)
    
    def __init__(self, context):
        self.context = context

    def checkName(self, name, object):
        return not self._getCheckId(object)(name, required=1)

    def chooseName(self, name, object):
        container = aq_inner(self.context)
        if not name:
            nameFromTitle = INameFromTitle(object, None)
            if nameFromTitle is not None:
                name = nameFromTitle.title
            if not name:
                name = getattr(aq_base(object), 'id', None)
            if not name:
                name = getattr(aq_base(object), 'portal_type', None)
            if not name:
                name = object.__class__.__name__

        if not isinstance(name, unicode):
            charset = self._getCharset(object)
            name = unicode(name, charset)

        request = getattr(object.__of__(container), 'REQUEST', None)
        if request is not None:
            name = IUserPreferredURLNormalizer(request).normalize(name)
        else:
            name = getUtility(IURLNormalizer).normalize(name)
            
        return self._findUniqueName(name, object)
            

    def _findUniqueName(self, name, object):
        """Find a unique name in the parent folder, based on the given id, by
        appending -n, where n is a number greater than 1, or just the id if
        it's ok.
        """
        check_id = self._getCheckId(object)

        if not check_id(name, required=1):
            return name

        idx = 1
        while idx <= ATTEMPTS:
            new_name = "%s-%d" % (name, idx)
            if not check_id(new_name, required=1):
                return new_name
            idx += 1

        raise ValueError("Cannot find a unique name based on %s after %d attemps." % (name, ATTEMPTS,))
        
    def _getCheckId(self, object):
        """Return a function that can act as the check_id script.
        """
        check_id = getattr(object, 'check_id', None)
        if check_id is None:
            parent = aq_inner(self.context)
            parent_ids = parent.objectIds()
            check_id = lambda id, required: id in parent_ids
        return check_id
        
    def _getCharset(self, object):
        """Returns the site default charset, or utf-8.
        """
        properties = queryUtility(IPropertiesTool)
        if properties is not None:
            site_properties = getattr(properties, 'site_properties', None)
            if site_properties is not None:
                return site_properties.getProperty('default_charset', 'utf-8')
        return 'utf-8'



from zope.component.testing import setUp, tearDown
import unittest, doctest

def test_suite():
    return unittest.TestSuite((
        doctest.DocFileSuite('namechooser.txt',
                             package='plone.app.content',
                             optionflags=doctest.ELLIPSIS,
                             setUp=setUp, tearDown=tearDown)))

if __name__ == "__main__":
    unittest.main(defaultTest='test_suite')



from urllib import quote_plus

from zope.component import getMultiAdapter, queryMultiAdapter, getAdapters, queryUtility

from zope.component.interfaces import IFactory
from zope.i18n import translate
from zope.app.container.constraints import checkFactory
from zope.app.publisher.interfaces.browser import AddMenu

from Acquisition import aq_inner
from Products.Five.browser import BrowserView

from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from Products.CMFPlone import PloneMessageFactory as _

from plone.i18n.normalizer.interfaces import IIDNormalizer

from plone.memoize.instance import memoize
from plone.memoize.request import memoize_diy_request

@memoize_diy_request(arg=0)
def _allowedTypes(request, context):
    return context.allowedContentTypes()

class FolderFactoriesView(BrowserView):
    """The folder_factories view - show addable types
    """
    
    def __call__(self):
        if 'form.button.Add' in self.request.form:
            url = self.request.form.get('url')
            self.request.response.redirect(url)
            return ''
        else:
            return self.index()
    
    def can_constrain_types(self):
        constrain_types = ISelectableConstrainTypes(self.add_context(), None)
        return constrain_types is not None and constrain_types.canConstrainTypes()
    
    @memoize
    def add_context(self):
        context_state = getMultiAdapter((self.context, self.request), name='plone_context_state')
        return context_state.folder()
    
    # NOTE: This is also used by plone.app.contentmenu.menu.FactoriesMenu.
    # The return value is somewhat dictated by the menu infrastructure, so
    # be careful if you change it

    def addable_types(self, include=None):
        """Return menu item entries in a TAL-friendly form.
        
        Pass a list of type ids to 'include' to explicitly allow a list of
        types.
        """
        context = aq_inner(self.context)
        request = self.request
        
        results = []
        
        portal_state = getMultiAdapter((context, request), name='plone_portal_state')
        portal_url = portal_state.portal_url()
        
        addContext = self.add_context()
        baseUrl = addContext.absolute_url()
        
        allowedTypes = _allowedTypes(request, addContext)
        
        # XXX: This is calling a pyscript (which we encourage people to customise TTW)
        exclude = addContext.getNotAddableTypes()

        # If there is an add view available, use that instead of createObject
        # Note: that this depends on the convention that the add view and the
        # factory have the same name, and it still only applies where there
        # is an FTI in portal_types to begin with. Alas, FTI-less content
        # is pretty much a no-go in CMF.
        
        addviews = {}
        addingview = queryMultiAdapter((addContext, request), name='+')
        if addingview is not None:
            for name, item in getAdapters((addingview, request), AddMenu):
                if item.extra:
                    factory_name = item.extra.get('factory')
                    if factory_name:
                        factory = queryUtility(IFactory, factory_name)
                        if factory and checkFactory(addContext, None, factory):
                            addviews[factory_name] = '%s/+/%s' % (baseUrl, item.action,)
        
        # XXX: Would be easier, but the add menu is not actually registered,
        # and this would depend on zope 3 security checks :-(
        # if addingview is not None:
        #     for item in addingview.addingInfo():
        #         factory = item['extra']['factory']
        #         addviews[factory] = '%s/+/%s' % (addContext.absolute_url(), item['action'],)

        idnormalizer = queryUtility(IIDNormalizer)
        for t in allowedTypes:
            typeId = t.getId()
            if typeId not in exclude and (include is None or typeId in include):
                cssId = idnormalizer.normalize(typeId)
                cssClass = 'contenttype-%s' % cssId
                factory = t.factory
                url = addviews.get(factory, '%s/createObject?type_name=%s' % (baseUrl, quote_plus(typeId),))
                icon = t.getIcon()
                if icon:
                    icon = '%s/%s' % (portal_url, icon)

                results.append({ 'title'        : t.Title(),
                                 'description'  : t.Description(),
                                 'action'       : url,
                                 'selected'     : False,
                                 'icon'         : icon,
                                 'extra'        : {'id' : cssId, 'separator' : None, 'class' : cssClass},
                                 'submenu'      : None,
                                })

        # Sort the addable content types based on their translated title
        results = [(translate(ctype['title'], context=request), ctype) for ctype in results]
        results.sort()
        results = [ctype[-1] for ctype in results]

        return results


from zope.interface import Interface
from zope import schema

class INameFromTitle(Interface):
    """An object that supports gettings it name from its title.
    """
    
    title = schema.TextLine(title=u"Title",
                            description=u"A title, which will be converted to a name",
                            required=True)

class IReindexOnModify(Interface):
    """Marker interface which makes sure an object gets reindexed when
    it's modified.
    """

class IBatch(Interface):
    """A batch splits up a large number of items over multiple pages"""

    size = schema.Int(title=u"The amount of items in the batch")

    firstpage = schema.Int(title=u"The number of the first page (always 1)")

    lastpage = schema.Int(title=u"The number of the last page")

    items_not_on_page = schema.List(
        title=u"All items that are in the batch but not on the current page")

    multiple_pages = schema.Bool(
        title=u"Boolean indicating wheter there are multiple pages or not")

    has_next = schema.Bool(
        title=u"Indicator for wheter there is a page after the current one")

    has_previous = schema.Bool(
        title=u"Indicator for wheter there is a page after the current one")

    previouspage = schema.Int(title=u"The number of the previous page")

    nextpage = schema.Int(title=u"The number of the nextpage page")

    next_item_count = schema.Int(title=u"The number of items on the next page")

    navlist = schema.List(
        title=u"List of page numbers to be used as a navigation list")

    show_link_to_first = schema.Bool(
        title=u"First page not in the navigation list")

    show_link_to_last = schema.Bool(
        title=u"Last page not in the navigation list")

    second_page_not_in_navlist = schema.Bool(
        title=u"Second page not in the navigation list")

    before_last_page_not_in_navlist = schema.Bool(
        title=u"Before last page not in the navigation list")

    islastpage = schema.Bool(
        title=u"Boolean indicating wheter the current page is the last page")

    previous_pages = schema.List(
        title=u"All previous pages that are in the navlist")

    next_pages = schema.List(
        title=u"All previous pages that are in the navlist")

class IIndexableObjectWrapper(Interface):
    """An adapter of a (object, portal) where object is to be indexed in 
    portal_catalog.
    
    This should implement __getattr__(), which in turn should react
    when the catalog tries to get attributes to index.
    
    The update() method must be called before the catalog is given the
    wrapper.
    """
    
    def update(vars, **kwargs):
        """Update the wrapper with variables from e.g. the workflow
        tool.
        """



from zope.component import getMultiAdapter
from zope.interface import implements

from AccessControl import Unauthorized
from Acquisition import aq_parent, aq_inner
from OFS.interfaces import IOrderedContainer
from Products.ATContentTypes.interface import IATTopic
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView

from plone.memoize import instance
from plone.app.content.browser.interfaces import IFolderContentsView
from plone.app.content.browser.tableview import Table
from kss.core import KSSView

from Products.CMFPlone.interfaces import IPloneSiteRoot

import urllib

NOT_ADDABLE_TYPES = ('Favorite',)

class FolderContentsView(BrowserView):
    """
    """
    implements(IFolderContentsView)
    
    def contents_table(self):
        table = FolderContentsTable(self.context, self.request)
        return table.render()

    def title(self):
        """
        """
        return self.context.pretty_title_or_id()

    def icon(self):
        """
        """
        ploneview = getMultiAdapter((self.context, self.request), name="plone")
        icon = ploneview.getIcon(self.context)
        return icon.html_tag()

    def parent_url(self):
        """
        """
        portal_url = getToolByName(self.context, 'portal_url')
        plone_utils = getToolByName(self.context, 'plone_utils')
        portal_membership = getToolByName(self.context, 'portal_membership')

        obj = self.context

        checkPermission = portal_membership.checkPermission

        # Abort if we are at the root of the portal
        if IPloneSiteRoot.providedBy(self.context):
            return None
        

        # Get the parent. If we can't get it (unauthorized), use the portal
        parent = aq_parent(aq_inner(obj))
        
        # # We may get an unauthorized exception if we're not allowed to access#
        # the parent. In this case, return None
        try:
            if getattr(parent, 'getId', None) is None or \
                   parent.getId() == 'talkback':
                # Skip any Z3 views that may be in the acq tree;
                # Skip past the talkback container if that's where we are
                parent = aq_parent(aq_inner(parent))

            if not checkPermission('List folder contents', parent):
                return None
    
            return parent.absolute_url()

        except Unauthorized:
            return None        

class FolderContentsKSSView(KSSView):
    def update_table(self, pagenumber='1', sort_on='getObjPositionInParent'):
        self.request.set('sort_on', sort_on)
        self.request.set('pagenumber', pagenumber)
        table = FolderContentsTable(self.context, self.request,
                                    contentFilter={'sort_on':sort_on})
        return self.replace_table(table)

    def replace_table(self, table):
        core = self.getCommandSet('core')
        core.replaceInnerHTML('#folderlisting-main-table', table.render())
        return self.render()

class FolderContentsTable(object):
    """   
    The foldercontents table renders the table and its actions.
    """                

    def __init__(self, context, request, contentFilter={}):
        self.context = context
        self.request = request
        self.contentFilter = contentFilter

        url = self.context.absolute_url()
        view_url = url + '/@@folder_contents'
        self.table = Table(request, url, view_url, self.items,
                           show_sort_column=self.show_sort_column,
                           buttons=self.buttons)

    def render(self):
        return self.table.render()

    @property
    @instance.memoize
    def items(self):
        """
        """
        plone_utils = getToolByName(self.context, 'plone_utils')
        plone_view = getMultiAdapter((self.context, self.request), name=u'plone')
        portal_workflow = getToolByName(self.context, 'portal_workflow')
        portal_properties = getToolByName(self.context, 'portal_properties')
        site_properties = portal_properties.site_properties
        
        use_view_action = site_properties.getProperty('typesUseViewActionInListings', ())
        browser_default = self.context.browserDefault()
                
        if IATTopic.providedBy(self.context):
            contentsMethod = self.context.queryCatalog
        else:
            contentsMethod = self.context.getFolderContents
        
        results = []
        for i, obj in enumerate(contentsMethod(self.contentFilter)):
            if (i + 1) % 2 == 0:
                table_row_class = "draggable even"
            else:
                table_row_class = "draggable odd"

            url = obj.getURL()
            path = obj.getPath or "/".join(obj.getPhysicalPath())
            icon = plone_view.getIcon(obj);
            
            type_class = 'contenttype-' + plone_utils.normalizeString(
                obj.portal_type)

            review_state = obj.review_state
            state_class = 'state-' + plone_utils.normalizeString(review_state)
            relative_url = obj.getURL(relative=True)
            obj_type = obj.portal_type

            modified = plone_view.toLocalizedTime(
                obj.ModificationDate, long_format=1)
            
            if obj_type in use_view_action:
                view_url = url + '/view'
            elif obj.is_folderish:
                view_url = url + "/folder_contents"              
            else:
                view_url = url

            is_browser_default = len(browser_default[1]) == 1 and (
                obj.id == browser_default[1][0])
                                 
            results.append(dict(
                url = url,
                id  = obj.getId,
                quoted_id = urllib.quote_plus(obj.getId),
                path = path,
                title_or_id = obj.pretty_title_or_id(),
                description = obj.Description,
                obj_type = obj_type,
                size = obj.getObjSize,
                modified = modified,
                icon = icon.html_tag(),
                type_class = type_class,
                wf_state = review_state,
                state_title = portal_workflow.getTitleForStateOnType(review_state,
                                                           obj_type),
                state_class = state_class,
                is_browser_default = is_browser_default,
                folderish = obj.is_folderish,
                relative_url = relative_url,
                view_url = view_url,
                table_row_class = table_row_class,
                is_expired = self.context.isExpired(obj),
            ))
        return results

    @property
    def orderable(self):
        """
        """        
        return IOrderedContainer.providedBy(self.context)

    @property
    def show_sort_column(self):
        return self.orderable and self.editable
        
    @property
    def editable(self):
        """
        """
        context_state = getMultiAdapter((self.context, self.request),
                                        name=u'plone_context_state')
        return context_state.is_editable()

    @property
    def buttons(self):
        buttons = []
        portal_actions = getToolByName(self.context, 'portal_actions')
        button_actions = portal_actions.listActionInfos(object=aq_inner(self.context), categories=('folder_buttons', ))

        # Do not show buttons if there is no data, unless there is data to be
        # pasted
        if not len(self.items):
            if self.context.cb_dataValid():
                for button in button_actions:
                    if button['id'] == 'paste':
                        return [self.setbuttonclass(button)]
            else:
                return []

        for button in button_actions:
            # Make proper classes for our buttons
            if button['id'] != 'paste' or self.context.cb_dataValid():
                buttons.append(self.setbuttonclass(button))
        return buttons

    def setbuttonclass(self, button):
        if button['id'] == 'paste':
            button['cssclass'] = 'standalone'
        else:
            button['cssclass'] = 'context'
        return button



from zope.component import getMultiAdapter

from Acquisition import aq_inner
from Products.Five import BrowserView
from Products.CMFCore.utils import getToolByName
from plone.app.content.browser.tableview import Table
import urllib

class FullReviewListView(BrowserView):

    def revlist(self):
        return self.context.my_worklist()

    def url(self):
        return self.context.absolute_url()+'/full_review_list'

    def review_table(self):
        table = ReviewListTable(self.context, self.request)
        return table.render()

class ReviewListTable(object):
    """   
    The review list table renders the table and its actions.
    """                

    def __init__(self, context, request):
        self.context = context
        self.request = request

        url = self.context.absolute_url()
        view_url = url + '/full_review_list'
        self.table = Table(request, url, view_url, self.items,
                           buttons=self.buttons)

    def render(self):
        return self.table.render()

    @property
    def items(self):
        plone_utils = getToolByName(self.context, 'plone_utils')
        portal_url = getToolByName(self.context, 'portal_url')
        plone_view = getMultiAdapter((self.context, self.request), name=u'plone')
        portal_workflow = getToolByName(self.context, 'portal_workflow')
        portal_properties = getToolByName(self.context, 'portal_properties')
        site_properties = portal_properties.site_properties
        
        use_view_action = site_properties.getProperty('typesUseViewActionInListings', ())
        browser_default = self.context.browserDefault()

        results = list()
        for i, obj in enumerate(self.context.my_worklist()):
            if i % 2 == 0:
                table_row_class = "even"
            else:
                table_row_class = "odd"
            
            url = obj.absolute_url()
            path = '/'.join(obj.getPhysicalPath())
            icon = plone_view.getIcon(obj);
            
            type_class = 'contenttype-' + plone_utils.normalizeString(
                obj.portal_type)

            review_state = portal_workflow.getInfoFor( obj, 'review_state', '')

            state_class = 'state-' + plone_utils.normalizeString(review_state)
            relative_url = portal_url.getRelativeContentURL(obj)
            obj_type = obj.portal_type

            modified = plone_view.toLocalizedTime(
                obj.ModificationDate(), long_format=1)
            
            if obj_type in use_view_action:
                view_url = url + '/view'
            elif obj.is_folderish:
                view_url = url + "/folder_contents"              
            else:
                view_url = url

            is_browser_default = len(browser_default[1]) == 1 and (
                obj.id == browser_default[1][0])
                                 
            results.append(dict(
                url = url,
                id  = obj.getId(),
                quoted_id = urllib.quote_plus(obj.getId()),
                path = path,
                title_or_id = obj.pretty_title_or_id(),
                description = obj.Description(),
                obj_type = obj_type,
                size = obj.getObjSize(),
                modified = modified,
                icon = icon.html_tag(),
                type_class = type_class,
                wf_state = review_state,
                state_title = portal_workflow.getTitleForStateOnType(review_state,
                                                           obj_type),
                state_class = state_class,
                is_browser_default = is_browser_default,
                folderish = obj.is_folderish,
                relative_url = relative_url,
                view_url = view_url,
                table_row_class = table_row_class,
                is_expired = self.context.isExpired(obj),
            ))
        return results

    @property
    def show_sort_column(self):
        return False

    def buttons(self):
        buttons = []
        portal_actions = getToolByName(self.context, 'portal_actions')
        button_actions = portal_actions.listActionInfos(object=aq_inner(self.context), categories=('folder_buttons', ))

        # Do not show buttons if there is no data, unless there is data to be
        # pasted
        if False:#not len(self.batch):
            if self.context.cb_dataValid():
                for button in button_actions:
                    if button['id'] == 'paste':
                        return [self.setbuttonclass(button)]
            else:
                return []

        for button in button_actions:
            # Make proper classes for our buttons
            if button['id'] != 'paste' or self.context.cb_dataValid():
                buttons.append(self.setbuttonclass(button))
        return buttons

    def setbuttonclass(self, button):
        if button['id'] == 'paste':
            button['cssclass'] = 'standalone'
        else:
            button['cssclass'] = 'context'
        return button



from zope.interface import implements

from zope.app.container.contained import Contained

from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl

from plone.app.content.interfaces import IReindexOnModify

class Item(PortalContent, DefaultDublinCoreImpl, Contained):
    """A non-containerish, CMFish item
    """
    
    implements(IReindexOnModify)
    
    def __init__(self, id=None, **kwargs):
        PortalContent.__init__(self, id, **kwargs)
        DefaultDublinCoreImpl.__init__(self, **kwargs)
        
        if id is not None:
            self.id = id


from zope.testing import doctest
from unittest import TestSuite

from Testing.ZopeTestCase import FunctionalDocFileSuite
from Products.PloneTestCase.PloneTestCase import FunctionalTestCase
from Products.PloneTestCase.PloneTestCase import setupPloneSite

setupPloneSite(extension_profiles=['Products.CMFPlone:testfixture'])

OPTIONFLAGS = (doctest.ELLIPSIS |
               doctest.REPORT_ONLY_FIRST_FAILURE |
               doctest.NORMALIZE_WHITESPACE)

class FolderTestCase(FunctionalTestCase):
    """base test case with convenience methods for all control panel tests"""

    def afterSetUp(self):
        super(FolderTestCase, self).afterSetUp()
        from Products.Five.testbrowser import Browser
        self.browser = Browser()
        
        self.uf = self.portal.acl_users
        self.uf.userFolderAddUser('root', 'secret', ['Manager'], [])

    def createDocuments(self, amount):
        self.setRoles(['Manager',])
        for i in range(1, amount):
            self.portal.invokeFactory(id='testing-%d' % i, type_name='Document')
            document = getattr(self.portal, 'testing-%d' % i)
            document.setTitle('Testing %d' % i)
            document.setExcludeFromNav(True)
            document.reindexObject()

    def createFolder(self, id='new-folder'):
        self.setRoles(['Manager',])
        self.portal.invokeFactory(id=id, type_name='Folder')
        folder = getattr(self.portal, id)
        folder.setTitle('New Folder')
        folder.setExcludeFromNav(True)
        folder.reindexObject()
        

    def loginAsManager(self):
        """points the browser to the login screen and logs in as user root with Manager role."""
        self.browser.open('http://nohost/plone/')
        self.browser.getLink('Log in').click()
        self.browser.getControl('Login Name').value = 'root'
        self.browser.getControl('Password').value = 'secret'
        self.browser.getControl('Log in').click()

def test_suite():
    tests = ['foldercontents.txt',]
    suite = TestSuite()
    for test in tests:
        suite.addTest(FunctionalDocFileSuite(test,
            optionflags=OPTIONFLAGS,
            package="plone.app.content.browser.tests",
            test_class=FolderTestCase))
    return suite



from Products.Five.browser.adding import ContentAdding
from Products.CMFCore.utils import getToolByName

class CMFAdding(ContentAdding):
    """An adding view with a less silly next-url
    """
    
    def add(self, content):
        content = super(CMFAdding, self).add(content)
        
        # We need to ensure that we finish type construction, not at least
        # to set the correct permissions based on the workflow
        portal_types = getToolByName(content, 'portal_types')
        fti = portal_types.getTypeInfo(content)
        if fti is not None:
            fti._finishConstruction(content)
        
        return content
    
    def nextURL(self):
        return "%s/%s/view" % (self.context.absolute_url(), self.contentName)


# Zope imports
from zope.interface import Interface
from zope.viewlet.interfaces import IViewletManager

class IFolderContentsView(Interface):
    """Interface, which provides methods for folder contens 
    """    
    def test(a, b, c):
        """A simple replacement of python's test.
        """
        
    def getAllowedTypes():
        """Returns allowed types for context.
        """
        
    def title():
        """Returns the title for the template.
        """
        
class IFolderContentsViewletManager(IViewletManager):
    """A viewlet manager for folder contents
    """



from zope.app.pagetemplate import ViewPageTemplateFile
from plone.app.content.batching import Batch
from plone.memoize import instance

class Table(object):
    """   
    The table renders a table with sortable columns etc.

    It is meant to be subclassed to provide methods for getting specific table info.
    """                

    def __init__(self, request, base_url, view_url, items, show_sort_column=False,
                 buttons=[]):
        self.request = request
        self.context = None # Need for view pagetemplate

        self.base_url = base_url
        self.view_url = view_url
        self.url = view_url
        self.items = items
        self.show_sort_column = show_sort_column
        self.buttons = buttons
        self.pagesize = 20

        selection = request.get('select')
        if selection == 'screen':
            self.selectcurrentbatch=True
        elif selection == 'all':
            self.selectall = True



    def set_checked(self, item):
        selected = self.selected(item)
        item['checked'] = selected and 'checked' or None
        item['table_row_class'] = item.get('table_row_class', '')
        if selected:
            item['table_row_class'] += ' selected'

    @property
    @instance.memoize
    def batch(self):
        b = Batch(self.items,
                  pagesize=self.pagesize,
                  pagenumber=int(self.request.get('pagenumber', 1)))
        map(self.set_checked, b)
        return b

    render = ViewPageTemplateFile("table.pt")
    batching = ViewPageTemplateFile("batching.pt")

    # options
    _selectcurrentbatch = False
    _select_all = False

    def _get_select_currentbatch(self):
        return self._selectcurrentbatch

    def _set_select_currentbatch(self, value):
        self._selectcurrentbatch = value
        if self._selectcurrentbatch and len(self.items) <= self.pagesize:
            self.selectall = True

    selectcurrentbatch = property(_get_select_currentbatch,
                                  _set_select_currentbatch)

    def _get_select_all(self):
        return self._select_all

    def _set_select_all(self, value):
        self._select_all = bool(value)
        if self._select_all:
            self._selectcurrentbatch = True

    selectall = property(_get_select_all, _set_select_all)

    @property
    def show_select_all_items(self):
        return self.selectcurrentbatch and not self.selectall

    def get_nosort_class(self):
        """
        """
        return "nosort"

    @property
    def selectall_url(self):
        return self.selectnone_url+'&select=all'

    @property
    def selectscreen_url(self):
        return self.selectnone_url+'&select=screen'

    @property
    def selectnone_url(self):
        pagenumber = self.request.get('pagenumber', '1')
        return self.view_url+'?pagenumber=%s'%pagenumber

    def selected(self, item):
        if self.selectcurrentbatch:
            return True
        return False

    @property
    def viewname(self):
        return self.view_url.split('?')[0].split('/')[-1]



import unittest, doctest

from Testing import ZopeTestCase as ztc

from base import ContentFunctionalTestCase

def test_suite():
    return unittest.TestSuite((
    
        ztc.ZopeDocFileSuite(
            'basecontent.txt', package='plone.app.content',
            test_class=ContentFunctionalTestCase,
            optionflags=(doctest.ELLIPSIS | 
                         doctest.NORMALIZE_WHITESPACE)),
    ))

if __name__ == "__main__":
    unittest.main(defaultTest='test_suite')



from zope import interface
from plone.app.content.interfaces import IBatch

class Batch(object):
    interface.implements(IBatch)

    def __init__(self, items, pagesize=20, pagenumber=1, navlistsize=5):
        self.items = items
        self.pagesize = pagesize
        self.pagenumber = pagenumber
        self.navlistsize = navlistsize

    def __len__(self):
        return len(self.items)

    def __iter__(self):
        start = (self.pagenumber-1) * self.pagesize
        end = start + self.pagesize
        return self.items[start:end].__iter__()

    @property
    def size(self):
        return len(self)

    @property
    def firstpage(self):
        return 1

    @property
    def lastpage(self):
        pages = self.size / self.pagesize
        if self.size % self.pagesize:
            pages += 1
        return pages

    @property
    def items_not_on_page(self):
        items_on_page = list(self)
        return [item for item in self.items if item not in
                items_on_page]

    @property
    def multiple_pages(self):
        return bool(self.size / self.pagesize)

    @property
    def has_next(self):
        return (self.pagenumber * self.pagesize) < self.size

    @property
    def has_previous(self):
        return self.pagenumber > 1

    @property
    def previouspage(self):
        return self.pagenumber - 1

    @property
    def nextpage(self):
        return self.pagenumber + 1

    @property
    def next_item_count(self):
        nextitems = self.size - (self.pagenumber * self.pagesize)
        if nextitems > self.pagesize:
            return self.pagesize
        return nextitems

    @property
    def navlist(self):
        start = self.pagenumber - (self.navlistsize / 2)
        if start < 1:
            start = 1
        end = start + self.navlistsize - 1
        if end > self.lastpage:
            end = self.lastpage

        return range(start, end+1)

    @property
    def show_link_to_first(self):
        return 1 not in self.navlist

    @property
    def show_link_to_last(self):
        return self.lastpage not in self.navlist

    @property
    def second_page_not_in_navlist(self):
        return 2 not in self.navlist

    @property
    def islastpage(self):
        return self.lastpage == self.pagenumber

    @property
    def previous_pages(self):
        return self.navlist[:self.navlist.index(self.pagenumber)]

    @property
    def next_pages(self):
        return self.navlist[self.navlist.index(self.pagenumber)+1:]

    @property
    def before_last_page_not_in_navlist(self):
        return (self.lastpage - 1) not in self.navlist

    @property
    def items_on_page(self):
        if self.islastpage:
            remainder = self.size % self.pagesize
            if remainder == 0:
                return self.pagesize
            else:
                return remainder
        else:
            return self.pagesize



from setuptools import setup, find_packages
import sys, os

version = '1.0'

setup(name='plone.app.content',
      version=version,
      description="Content Views for Plone",
      long_description="""\
plone.app.content contains various views for Plone, such as 
folder_contents, as well as general content infrastructure, such as
base classes and name choosers.
""",
      classifiers=[
          "Framework :: Plone",
          "License :: OSI Approved :: GNU General Public License (GPL)",
          "Programming Language :: Python",
      ],
      keywords='plone content views viewlet',
      author='Jeroen Vloothuis',
      author_email='jeroen.vloothuis@xs4all.nl',
      url='http://svn.plone.org/svn/plone/plone.app.content',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['plone.app'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'setuptools',
        'plone.i18n',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      )
